/*********************************************************************************

Copyright(c) 2015 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*
 * Project		MCAPI+MDMA example - Dual Sharc Talkthrough
 * Description	The purpose of this project is to demonstrate the MCAPI and MDMA
 * 				Inter-Processor Communications (IPC) methods. The project takes
 * 				four channels of audio from the ADAU1979 CODEC into
 * 				Sharc 0 (core 1) L1 SRAM and sends the data directly to
 * 				Sharc 1 (core 2) L1 SRAM using MDMA during runtime. MCAPI is used
 * 				to initialize and synchronize the MDMA channel during initialization.
 * 				The ARM is used during initialization but is not used during runtime.
 *
 * File
 * Description	The two Sharcs come out of reset at approximately the same time.
 * 				This file conatins main() for Sharc 1. Sharc 1 manages the ADC.
 * 				Four channels of audio data are DMA'd into L1. The AdcCallback()
 * 				is executed for each frame of audio from the CODEC. The frame is
 * 				passed through a filter and the result is copied directly into
 * 				Sharc 2 L1 SRAM using MDMA.
 *
 */
#include <sys/platform.h>
#include <sys/adi_core.h>
#include <stdio.h>
#include <drivers/adc/adau1979/adi_adau1979.h>
#include "adi_initialize.h"
#include "MultiCoreTalkThruSharcSharc_Core1.h"
#include "SHARC_linkInterface.h"

//*****************************************************************************
// External Functions
//*****************************************************************************
extern uint32_t Adau1979Init(void);
extern uint32_t Adau1979SubmitBuffers(void);
extern uint32_t Adau1979Enable( void );
extern uint32_t Adau1979DoneWithBuffer( void *pBuffer );
extern void InputSeriesFilter( int8_t *InputBuffer, int8_t *OutputBuffer, uint32_t BytesToProcess );


//*****************************************************************************
// Local Variables
//*****************************************************************************
/* Counter to keep track of number of ADC buffers processed */
uint32_t AdcCount = 0u;
int8_t BufferToSendToCore2[AUDIO_BUFFER_SIZE];
void *DMASlaveDestinationAddress;

/** 
 * If you want to use command program arguments, then place them in the following string. 
 */
char __argv_string[] = "";

int main(int argc, char *argv[])
{
	uint32_t DestAddress;

	/**
	 * Initialize managed drivers and/or services that have been added to 
	 * the project.
	 * @return zero on success 
	 */
	adi_initComponents();
	
	// Initialise ADAU1979
	if(Adau1979Init() != 0u)
	{
		fprintf(stdout, "Core1: Adau1979Init() failed\n" );
		while(1){asm(" NOP; ");}
	}

	// Submit ADC buffers
	if(Adau1979SubmitBuffers() != 0u)
	{
		fprintf(stdout, "Core1: Adau1979SubmitBuffers() failed\n" );
		while(1){asm(" NOP; ");}
	}

	adi_core_enable(ADI_CORE_SHARC1); // EMG - 071116 - Moved to SHARC1 to avoid SHARC2 starting before SHARC1

	// Initialize SHARClink
	// The code will block here until the other core executes.
	if(SHARC_linkMasterInit(&DestAddress) != 0u)
	{
		fprintf(stdout, "Core1: SHARClinkMasterInit() failed\n" );
		while(1){asm(" NOP; ");}
	}

	DMASlaveDestinationAddress = (void *)DestAddress;

	fprintf(stdout, "Starting ADC/DAC\n" );// EMG - 071116 - Moved to avoid sync issues

	// Enable data flow for only the ADC, DAC is enabled only after ping pong buffers of ADC are filled.
	if(Adau1979Enable() != 0u)
	{
		fprintf(stdout, "Core1: Adau1979Enable() failed\n" );
		while(1){asm(" NOP; ");}
	}


	// Processing is done in callback
	while(1)
	{
		asm(" NOP; ");
	}

	return 0;
}


/*
 * ADC Callback.
 *
 * Parameters
 *  None
 *
 * Returns
 *  None
 *
 */
void AdcCallback(void *pCBParam, uint32_t nEvent, void *pArg)
{
    switch(nEvent)
    {
        case ADI_SPORT_EVENT_RX_BUFFER_PROCESSED:
        	AdcCount++;
        	// Run filter on audio buffer
        	InputSeriesFilter( pArg, &BufferToSendToCore2[0], AUDIO_BUFFER_SIZE );

        	// Send filtered audio to other SHARC
        	if( SHARC_linkSend( (void *)&BufferToSendToCore2[0], DMASlaveDestinationAddress, 1, AUDIO_BUFFER_SIZE ) != 0 )
        	{
        		while(1){asm(" NOP; ");}
        	}

        	// Return buffer to pool
        	Adau1979DoneWithBuffer( pArg );
        	break;
        default:
            break;
    }
}


