/*********************************************************************************

Copyright(c) 2015 Analog Devices, Inc. All Rights Reserved.

This software is proprietary and confidential.  By using this software you agree
to the terms of the associated Analog Devices License Agreement.

*********************************************************************************/
/*
 * Project		MCAPI+MDMA example - Dual Sharc Talkthrough
 * Description	The purpose of this project is to demonstrate the MCAPI and MDMA
 * 				Inter-Processor Communications (IPC) methods. The project takes
 * 				four channels of audio from the ADAU1979 CODEC into
 * 				Sharc 0 (core 1) L1 SRAM and sends the data directly to
 * 				Sharc 1 (core 2) L1 SRAM using MDMA during runtime. MCAPI is used
 * 				to initialize and synchronize the MDMA channel during initialization.
 * 				The ARM is used during initialization but is not used during runtime.
 *
 * File
 * Description	In order to use MDMA to transfer data between cores, the driver needs
 * 				to be setup in a particular sequence. Sharc 1 is the source or master
 * 				core. It opens the MDMA channel on a specific stream, then uses MCAPI
 * 				to send the Sid to Sharc 2. Sharc 2 is the destination or slave core.
 * 				After receiving the Sid, the slave core installs an interrupt handler
 * 				to handle the MDMA completion event. Audio frames cannot be sent to the
 * 				slave core until the slave sends a message to the master core saying the
 * 				interrupt handler is installed. After the slave core installs its interrupt
 * 				handler, it sends the destination buffer address to the master core. After
 * 				the master core receives the destination buffer address, it starts sending
 * 				audio frames to the destination buffer address.
 *
 */
#include <services/dma/adi_dma.h>
#include <services/mcapi/mcapi.h>
#include <services/int/adi_sec.h>
#include "SHARC_linkInterface.h"
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

/*=============  MCAPI    =============*/
/* MCAPI Domain number. Zero is the only supported domain at present. */
#define DOMAIN 0u
#define NODE_CORE_0 1u
#define NODE_CORE_1 2u


/*=============  D A T A  =============*/

/* DMA Stream Handle */
ADI_DMA_STREAM_HANDLE   hMemDmaStream;
/* Source DMA Handle */
ADI_DMA_CHANNEL_HANDLE  hSrcDmaChannel;
/* Destination DMA Handle */
ADI_DMA_CHANNEL_HANDLE  hDestDmaChannel;

/* Memory to handle DMA Stream */
static uint8_t MemDmaStreamMem[ADI_DMA_STREAM_REQ_MEMORY];


static void MemDmaCallback(void *pCBParam, uint32_t Event, void *pArg)
{
    /* CASEOF (Event) */
    switch (Event)
    {
    	/* CASE (Processed a one-shot/circular buffer) */
    	case (ADI_DMA_EVENT_BUFFER_PROCESSED):
    	    /* Update memory copy status flag */
//			bMemCopyInProgress = false;
    		break;
    }
}

//*****************************************************************************
// Called from Master Sharc (sending Sharc) to initialize the SHARC link.
// Uses MCAPI to establish SHARC link. SHARC link does not use MCAPI during runtime.
//
// 1) Master opens MDMA channel
// 2) Master configures MDMA channel
// 3) Master opens MCAPI connection to slave
// 4) Master sends Sid to slave
// 5) Slave waits for Sid from Master
// 6) Slave inserts interrupt handler based on Sid
// 7) Slave sends DMA destination buffer address to Master
// 8) Master waits for buffer address from slave
//
// After the SHARC link connection is established, DMA transfers are initiated
// by calling SHARC_linkSend().
//*****************************************************************************
int SHARC_linkMasterInit( uint32_t *DMASlaveDestinationAddress )
{
	uint32_t nSid;

	// MCAPI endpoints
	mcapi_endpoint_t 	master_ep;							// SHARClink master endpoint (this core)
	mcapi_endpoint_t 	slave_ep;							// SHARClink slave endpoint (other core)
	size_t				RcvBytes;
	uint32_t			DestAddress;
    ADI_DMA_RESULT      eResult = ADI_DMA_SUCCESS;			// DMA return code
    mcapi_status_t 		mcapi_status = MCAPI_ERR_GENERAL;	// MCAPI return code


#ifdef MCAPI_VERBOSE
	char status_buff[MCAPI_MAX_STATUS_MSG_LEN];				// Buffer for MCAPI verbose error reporting
#endif

	//*************************************************************************
	// Open MDMA channel
	//*************************************************************************
    DEBUGMSG(stdout, "Core1: Opening MDMA channel\n" );
    eResult = adi_mdma_Open (MEMCOPY_STREAM_ID,
                             &MemDmaStreamMem[0],
                             &hMemDmaStream,
                             &hSrcDmaChannel,
                             &hDestDmaChannel,
                             NULL,
                             NULL);

    if (eResult != ADI_DMA_SUCCESS)
    {
    	DEBUGMSG(stdout,"Failed to open MDMA stream, Error Code: 0x%08X\n", eResult);
    	return SHARC_LINK_ERROR;
    }

	//*************************************************************************
	// Configure MDMA channel
	//*************************************************************************
    adi_mdma_EnableChannelInterrupt(hDestDmaChannel,false,false);	// Disable the MDMA destination transfer complete interrupt
    adi_mdma_GetChannelSID (hDestDmaChannel,&nSid);					// Get the channel SID for the MDMA destination complete interrupt
    adi_sec_SetCoreID(nSid, ADI_SEC_CORE_1);						// Set interrupt to occur on Core 2 (unfortunate enumeration name in driver)
    adi_mdma_EnableChannelInterrupt(hSrcDmaChannel,true,true);		// Enable the MDMA source transfer complete interrupt
    DEBUGMSG(stdout,"\n Destination Channel interrupt SID: \t%08X\n ",nSid);
#if 1
    eResult = adi_dma_UpdateCallback (hSrcDmaChannel, MemDmaCallback, hMemDmaStream); // Register source transfer complete interrupt
     /* IF (Failure) */
     if (eResult != ADI_DMA_SUCCESS)
     {
         DEBUGMSG("Failed to set DMA callback, Error Code: 0x%08X\n", eResult);
         return SHARC_LINK_ERROR;
     }
#endif
	//*************************************************************************
	// Open MCAPI connection to slave
	//*************************************************************************
    DEBUGMSG(stdout, "Core1: Creating MCAPI master endpoint\n" );
    master_ep = mcapi_endpoint_create(1, &mcapi_status);	// Create master MCAPI endpoint
    if (MCAPI_SUCCESS != mcapi_status)
    {
  #ifdef MCAPI_VERBOSE
    	mcapi_display_status(mcapi_status, status_buff, sizeof(status_buff));
    	DEBUGMSG(stdout, "[ERR] mcapi_endpoint_create %s %s %d\n", status_buff, __FILE__, __LINE__);
  #endif
    	return(SHARC_LINK_MCAPI_ERROR);
    }

    DEBUGMSG(stdout, "Core1: Waiting for MCAPI slave endpoint creation\n" );
    slave_ep = mcapi_endpoint_get(DOMAIN, NODE_CORE_1, 1, MCAPI_TIMEOUT_INFINITE, &mcapi_status);
    if (MCAPI_SUCCESS != mcapi_status)						// Get MCAPI slave endpoint
    {
  #ifdef MCAPI_VERBOSE
    	mcapi_display_status(mcapi_status, status_buff, sizeof(status_buff));
    	DEBUGMSG(stdout, "[ERR] mcapi_endpoint_get %s %s %d\n", status_buff, __FILE__, __LINE__);
  #endif
    	return(SHARC_LINK_MCAPI_ERROR);
    }
    DEBUGMSG(stdout, "Core1: MCAPI slave endpoint captured\n" );

	//*************************************************************************
	// Send SID to slave
	//*************************************************************************
    DEBUGMSG(stdout, "Core1: Sending Sid to slave\n" );
    mcapi_msg_send(master_ep, slave_ep, &nSid, sizeof(nSid), 0, &mcapi_status);
    if (MCAPI_SUCCESS != mcapi_status)						// Send Sid to slave over MCAPI
    {
  #ifdef MCAPI_VERBOSE
    	mcapi_display_status(mcapi_status, status_buff, sizeof(status_buff));
    	DEBUGMSG(stdout, "[ERR] mcapi_msg_send_i %s %s %d\n", status_buff, __FILE__, __LINE__);
  #endif
    	return(SHARC_LINK_MCAPI_ERROR);
    }

	//*************************************************************************
	// Wait for buffer address from slave
	//*************************************************************************
    DEBUGMSG(stdout, "Core1: Waiting for destination address from slave\n" );
	mcapi_msg_recv(master_ep, &DestAddress, sizeof(DestAddress), &RcvBytes, &mcapi_status);
	if ( MCAPI_SUCCESS != mcapi_status )					// Block here until we receive the buffer address from slave
	{
#ifdef MCAPI_VERBOSE
		mcapi_display_status(mcapi_status, status_buff, sizeof(status_buff));
	    DEBUGMSG(stdout, "[ERR] mcapi_msg_recv %s %s %d\n", status_buff, __FILE__, __LINE__);
#endif
	    return(SHARC_LINK_MCAPI_ERROR);
	}

	if( RcvBytes != 4 ) return(SHARC_LINK_MCAPI_ERROR);		// Error if we did not receive the address

	//*************************************************************************
	// SHARC_link connection established
	//*************************************************************************
	DEBUGMSG(stdout, "Core1: SHARKlink connection established 0x%08X\n",  DestAddress );
	*DMASlaveDestinationAddress = DestAddress;				// Return DMA destination address

    return SHARC_LINK_SUCCESS;
}


//*****************************************************************************
// Send a buffer to the slave core using MDMA. Upon transfer completion, an
// interrupt is generated on the slave core.
//*****************************************************************************
int SHARC_linkSend( void *pSrcBuffer, void *pDestBuffer, uint8_t nBytesInElement, uint32_t nElementsInBuffer )
{
	ADI_DMA_MSIZE		ElementSize;
    /* DMA Return code */
    ADI_DMA_RESULT      eResult = ADI_DMA_SUCCESS;

    switch( nBytesInElement )								// Set ElementSize to 1,2, or 4 bytes
    {
    	case 1: ElementSize = ADI_DMA_MSIZE_1BYTE;  break;
    	case 2: ElementSize = ADI_DMA_MSIZE_2BYTES; break;
    	case 4: ElementSize = ADI_DMA_MSIZE_4BYTES; break;
    	default: ElementSize = 0; return -1;
    }

    /* Submit One-shot 1D buffers to Source and destination channel */
    eResult = adi_mdma_Copy1D (hMemDmaStream,				// initiate MDMA transfer
                              (void *)pDestBuffer,
                              (void *)pSrcBuffer,
                              ElementSize,
                              (nElementsInBuffer / nBytesInElement));

    /* IF (Failure) */
    if (eResult != ADI_DMA_SUCCESS)
    {
        DEBUGMSG(stdout,"Failed to initiate One-shot 1D memory copy, Error Code: 0x%08X\n", eResult);
        return SHARC_LINK_ERROR;
    }

    return SHARC_LINK_SUCCESS;
}

